function initME = initEngine(conScale,apuMode)

minVd = 0.5;    % Minimum displacement
maxVd = 5.0;    % Maximum displacement
rangeVd = maxVd - minVd;

Vd = (conScale/100)*rangeVd;
Vd = Vd + minVd;

switch apuMode
    case 1  % Non-APU mode
        initME.engine = eng(0,Vd,apuMode);
        initME.costA = initME.engine.cost;
        initME.massA = initME.engine.mass;
        
    case 2  % APU mode
        initME.engine = eng(0,Vd,apuMode);
        
    otherwise
        disp('Invalid engine mode selection');
end

end


function eng = eng(toPlot,Vd,apuMode)

% Description
%
% ne = we * Te / Pc
%
% mf_dot = Pc / Hl
%
% cm = we * S / pi
%
% pme = N * pi * Te / Vd
%
% Pe = z * pi / 16 * B ^ 2 * pme * cm ?
%
% pme = e * pmf - pme0

eng.Vd = Vd;

Vd = Vd*0.001;              % Convert litres to m^3

S = 0.067;                  % Stroke length (m)

Hl = 44.4 * 10 ^ 6;         % Fuel lower heating value (J/kg)

cm = linspace(0,20,100);    % Mean piston speed (m/s)

pme = linspace(0,15*10^5,200);  % Piston mean effective pressure (Pa)

ne = zeros(length(pme),length(cm));     % Engine efficiency

pout = zeros(length(pme),length(cm));   % Engine power (W)

mf_dot = zeros(length(pme),length(cm)); % Fuel flow rate (kg/s)

ecm = zeros(length(cm),1);      % Efficiency along cm

pme0cm = zeros(length(cm),1);   % Engine losses (-Pa)

we = zeros(length(cm),1);       % Engine speed (rad/s)

Te = zeros(length(pme),1);      % Engine Torque (Nm)

for i = 1:length(pme)
    for j = 1:length(cm)
        we(j) = cm(j) * pi / S;
        Te(i) = pme(i) * Vd / (4 * pi);
        
        switch apuMode
            case 1  % Non-APU mode
                ecm(j) = calce(cm(j));
                
            case 2  % APU mode
                ecm(j) = calce(cm(j)).*limitEngine(pme(i),cm(j),apuMode);
                
            otherwise
                disp('Invalid engine mode selection');
        end
        
        pme0cm(j) = calcpme0(cm(j));
        pmf = ( pme(i) + pme0cm(j) ) / ecm(j);
        
        if i > 1
            mf_dot(i,j) = pmf * we(j) * Vd / (4 * pi *Hl);
        else
            mf_dot(i,j) = 0;
        end
        pout(i,j) = we(j) * Te(i);
    end
end

ne = pout ./ (mf_dot .* Hl); % Pc = mf_dot(i,j) * Hl;

eng.ne = ne;
eng.pout = pout;
eng.we = we;
eng.Te = Te;
eng.Hl = Hl;
eng.mf_dot = mf_dot;
eng.enginePow = max(pout(ne>0.1));

if apuMode == 1  % Non-APU mode
    eng.Telimit = limitEngine(Vd,cm,apuMode);
    eng.mass = 67.6 * Vd * 1000 * 1.2;
    eng.cost = 14.50 * eng.enginePow / 1000 + 531;
end

if toPlot
    close all;
    figure(10);
    
    subplot(2,2,1);
    grid on;
    [C h] = contour(cm,pme,ne.*100);
    clabel(C,h);
    xlabel('Mean Piston Speed (m/s)');
    ylabel('Mean Effective Pressure (Pa)');
    title('Power (kW) / Efficiency (%)');
    hold on;
    %[C h] = contour(cm,pme,pout./1000,10:10:200);
    %clabel(C,h);
    
    subplot(2,2,3);
    grid on;
    [AX,H1,H2] = plotyy(cm,-pme0cm,cm,ecm);
    set(get(AX(1),'Ylabel'),'String','pme0');
    set(get(AX(2),'Ylabel'),'String','e');
    set(get(AX(1),'Xlabel'),'String','Mean Piston Speed (m/s)');
    
    subplot(2,2,2);
    [C h] = contour(we,Te,mf_dot.*60.*60,1:1:100);
    clabel(C,h);
    xlabel('Speed (rad/s)');
    ylabel('Torque(Nm)');
    title('Fuel Flow (kg/s)')
    
    subplot(2,2,4);
    grid on;
    [C h] = contour(we,Te,mf_dot.*1000.*60.*60.*1000./pout,[100:50:350 400:100:600]);
    clabel(C,h);
    xlabel('Speed (rad/s)');
    ylabel('Torque(Nm)');
    title('Brake Specific Fuel Consumption (g/kWh)');
    hold on;
    
    figure(20);
    grid on;
    hold on;
    plot(eng.we,eng.Telimit);
    xlabel('Speed (rad/s)');
    ylabel('Torque(Nm)');
    title('Engine Peak Torque (Nm)');
end

end


function e = calce(cm)
%e = ((-(cm-12).^2+400)-256)/144*0.05+0.37;
fitPoly = [0.0000   -0.0004    0.0097    0.3614];
e = polyval(fitPoly,cm);
end


function pme0 = calcpme0(cm)
%pme0 = 2 * 10 ^ 5 + (cm*cm/400 * 2 * 10 ^ 5);
fitPoly = [0.0005   -0.0069   0.1283   1.0239].*1e5;
pme0 = polyval(fitPoly,cm);
end


function p = limitEngine(pme_or_Vd,cm,apuMode)

switch apuMode
    case 1  % Non-APU mode
        fitPoly = [0.0005   -0.0320    0.5427    7.8219].*1e5;
        pme = polyval(fitPoly,cm);
        pme(cm>17.65) = 0;
        pme(cm<2.23) = 0;
        p = pme .* pme_or_Vd / (4 * pi);
        
    case 2  % APU mode
        fitPoly = [0.0005   -0.0320    0.5427    7.8219].*1e5;
        if cm > 17.65
            p = 0.01;
        elseif cm < 2.23
            p = 0.01;
        elseif pme_or_Vd > polyval(fitPoly,cm);
            p = 0.01;
        else
            p=1;
        end
        
    otherwise
        disp('Invalid engine mode selection');
end
end
