function [in simu] = runModel(vehMass,massLimit,drivecycleType,drivecycleRepeat,fdRatio,conScale,stoScale,conScale2,stoScale2,costLimit,powertrainVariant,powertrainSelection)

if ~nargin
    vehMass = [];   % Vehicle mass
    massLimit = [];
    drivecycleType = [];
    drivecycleRepeat = [];
    fdRatio = [];
    conScale = [];  % Scale of primary energy converter from 0 - 100
    stoScale = [];  % Scale of primary energy storage from 0 - 100
    conScale2 = [];  % Scale of secondary energy converter from 0 - 100
    stoScale2 = [];  % Scale of secondary energy storage from 0 - 100
    costLimit = [];
    powertrainVariant = [];
    powertrainSelection = [];
end


%% *** INITIALISE SIGNATURE ***

if isempty(vehMass)
    vehMass = 1000;
end

if isempty(massLimit)
    massLimit = 10000;
end

% Drivecycle to Use
% 0 - Blank
% 1 - NEDC
% 2 - Real - World
% 3 - Artemis
% 4 - Constant Speed where type is speed value

if isempty(drivecycleType)
    drivecycleType = 1;
end

if isempty(fdRatio)
    fdRatio = 5;
end

if isempty(conScale)
    conScale = 50;
end

if isempty(stoScale)
    stoScale = 50;
end

if isempty(conScale2)
    conScale2 = 50;
end

if isempty(stoScale2)
    stoScale2 = 50;
end

if isempty(costLimit)
    costLimit = 1e9;
end

if isempty(powertrainSelection)
    powertrainSelection = 1:8; % Remember to adjust maximum powertrain variants here
end

if isempty(powertrainVariant)
    powertrainVariant = 1;
end

if checkRoundNumbers([conScale stoScale conScale2 stoScale2 powertrainVariant]) == 0
    keyboard;
end


%% *** MAP POWERTRAIN SELECTION TO POWERTRAIN VARIANTS ***

initVariant; % Remember to adjust maximum powertrain variants here
Powertrain = powertrainSelection(powertrainVariant);


%% *** INITIALISE VARIABLES ***

in = getDrivecycle(drivecycleType,[],drivecycleRepeat);
in.vehMass = vehMass;

in.sim_stoptime = max(in.dc_time);

in.batIniSOC = 89;  % Initial battery SOC (%)
in.batIniSOH = 100; % Initial battery SOH (%)
in.fd = fdRatio;
in.mu_aero = 0.3;
in.veh_wheelr = 0.3704;
in.sim_ts = 1;
in.busVoltage = 400;

%Probably deserves its own function call in the future when more elaborate controllers are created
in.thermoControlPoint = 30;
in.vehMode = 0; % Used for EV to switch between charge/running a drivecycle


%% *** INITIALISE COMPONENTS ***

% Powertrain Selection
% 1 - Conventional Powertrain
% 2 - EV
% 3 - Series PHEV
% 4 - Fuel Cell + Ultracap
% 5 - Parallel Flywheel + CVT
% 6 - Battery + Ultracap
% 7 - EV (with SOH)
% 8 - Battery (with SOH) + Ultracap

switch Powertrain
    case 1
        inA = initEngine(conScale,1);
        inB = initTank(stoScale,1);
        inC = initGearbox(in.dc_speed.*3.6,1);
        
    case 2
        inA = initMachine(conScale,1,[]);
        inB = initBattery(stoScale,in.batIniSOC,1,0,[]);
        inC = initGearbox(in.dc_speed.*3.6,2);
        
    case 3
        inA = initMachine(conScale,1,[]);
        inB = initBattery(stoScale,in.batIniSOC,1,0,[]);
        inC = initGearbox(in.dc_speed.*3.6,2);
        inD = initGenSet(conScale2);
        inE = initTank(stoScale2,2);
        
    case 4
        inA = initMachine(conScale,1,[]);
        inB = initUltraCap(stoScale);
        inC = initGearbox(in.dc_speed.*3.6,2);
        inD = initFuelCell(conScale2);
        inE = initTankFC(stoScale2);
        
    case 5
        inA = initEngine(conScale,1);
        inB = initTank(stoScale,1);
        inC = initGearbox(in.dc_speed.*3.6,1);
        inD = initCVT(conScale2);
        inE = initFlywheel(stoScale2);
        
    case 6
        inA = initMachine(conScale,1,50);
        inB = initUltraCap(stoScale);
        inC = initGearbox(in.dc_speed.*3.6,2);
        inD = initNull_con(conScale2);
        inE = initBattery(stoScale2,in.batIniSOC,2,0,[]);
        
    case 7
        inA = initMachine(conScale,1,[]);
        inB = initBattery(stoScale,in.batIniSOC,1,1,in.batIniSOH);
        inC = initGearbox(in.dc_speed.*3.6,2);
        
    case 8
        inA = initMachine(conScale,1,50);
        inB = initUltraCap(stoScale);
        inC = initGearbox(in.dc_speed.*3.6,2);
        inD = initNull_con(conScale2);
        inE = initBattery(stoScale2,in.batIniSOC,2,1,in.batIniSOH);
        
    otherwise
        disp('Invalid powertrain selection');
end

inTemp = in;
switch Powertrain
    case {1,2,7}
        in = mergeStruct(inTemp,inA,inB,inC);
        in.costT = in.costA + in.costB + in.costC;
        in.massT = in.vehMass + in.massA + in.massB + in.massC;
        in.hybridType = 0;
    case 5
        in = mergeStruct(inTemp,inA,inB,inC,inD,inE);
        in.costT = in.costA + in.costB + in.costC + in.costD + in.costE;
        in.massT = in.vehMass + in.massA + in.massB + in.massC + in.massD + in.massE;
        in.hybridType = 1;
    otherwise
        in = mergeStruct(inTemp,inA,inB,inC,inD,inE);
        in.costT = in.costA + in.costB + in.costC + in.costD + in.costE;
        in.massT = in.vehMass + in.massA + in.massB + in.massC + in.massD + in.massE;
        in.hybridType = 0;
end


%% *** RUN SIMULATION ***

if in.costT < costLimit
    if in.massT < massLimit
        simu = runSimu(in,Powertrain);
    end
end


%% *** POST-PROCESSING ***
% keyboard;

if in.costT < costLimit
    if in.massT < massLimit
        simu.distance = trapz(simu.t,simu.speed);
        
        switch Powertrain
            case 1
                simuA = simuEngine(in,simu);
                
            case {2,6}
                simuA = simuMachine(in,simu,Powertrain);
                
            case 3
                simuA = simuGenSet(in,simu,Powertrain);
                
            case 4
                simuA = simuFuelCell(in,simu,Powertrain);
                
            case 5
                simuA = simuFlywheel(in,simu);
                
            case {7,8}
                simuA = simuSOH(in,simu,Powertrain);
                
            otherwise
                disp('Invalid powertrain post-processing');
        end
        
        simuTemp = simu;
        simu = mergeStruct(simuTemp,simuA);
    else
        simu.wtw_CO2 = inf;
        simu.ttw_CO2 = inf;
    end
else
    simu.wtw_CO2 = inf;
    simu.ttw_CO2 = inf;
end

% keyboard;
end


function confirmRound = checkRoundNumbers(inputArray)
% Redundant check to ensure that inputs are round numbers
confirmRound = 1;

for i = 1:length(inputArray)
    if inputArray(i) ~= round(inputArray(i))
        disp('Error: inputs are not integer values!');
        confirmRound = 0;
    end
end
end
